@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003, 2004 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 
 #include <config.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <string.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
@@ -33,6 +34,7 @@
 #include <exif-data.h>
 #include <exif-content.h>
 #include <exif-entry.h>
+#include <exif-utils.h>
 #include "jpegutils/jpeg-data.h"
 #endif /* HAVE_LIBEXIF */
 
@@ -40,17 +42,19 @@
 #include "gconf-utils.h"
 #include "gthumb-window.h"
 #include "gtk-utils.h"
-#include "image-viewer.h"
+#include "image-loader.h"
 #include "main.h"
 #include "icons/pixbufs.h"
 #include "jpegutils/jpegtran.h"
 #include "pixbuf-utils.h"
+#include "gthumb-stock.h"
+#include "gth-exif-utils.h"
 
 
 #define ROTATE_GLADE_FILE "gthumb_tools.glade"
 #define PROGRESS_GLADE_FILE "gthumb_png_exporter.glade"
 
-#define PREVIEW_SIZE 256
+#define PREVIEW_SIZE 200
 
 enum {
 	TRAN_ROTATE_0,
@@ -68,14 +72,19 @@ typedef struct {
 
 	GtkWidget    *dialog;
 	GtkWidget    *j_button_box;
+	GtkWidget    *j_button_vbox;
+	GtkWidget    *j_revert_button;;
 	GtkWidget    *j_apply_to_all_checkbutton;
+	GtkWidget    *j_preview_image;
+	GtkWidget    *j_from_exif_checkbutton;
 
 	int           rot_type;    /* 0, 90 ,180, 270 */
 	int           tran_type;   /* mirror, flip */
 	GList        *file_list;
 	GList        *files_changed_list;
 	GList        *current_image;
-	GtkWidget    *viewer;
+	ImageLoader  *loader;
+	GdkPixbuf    *original_preview;
 } DialogData;
 
 
@@ -92,6 +101,7 @@ destroy_cb (GtkWidget  *widget,
 	all_windows_add_monitor ();
 
 	file_data_list_free (data->file_list);
+	g_object_unref (data->loader);
 	g_object_unref (data->gui);
 	g_free (data);
 }
@@ -111,11 +121,162 @@ add_image_to_button (GtkWidget    *button,
 }
 
 
-static void
-image_loaded_cb (ImageViewer *viewer,
-		 DialogData  *data)
+static GdkPixbuf *
+_gdk_pixbuf_scale_keep_aspect_ratio (GdkPixbuf *pixbuf, 
+				     int        max_width, 
+				     int        max_height)
 {
-	gtk_widget_set_sensitive (data->j_button_box, TRUE);
+	int        width, height;
+	GdkPixbuf *result = NULL;
+
+	if (pixbuf == NULL)
+		return NULL;
+
+	width = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+	if (scale_keepping_ratio (&width, &height, max_width, max_height))
+		result = gdk_pixbuf_scale_simple (pixbuf, width, height, GDK_INTERP_BILINEAR);
+	else {
+		result = pixbuf;
+		g_object_ref (result);
+	}
+
+	return result;
+}
+
+
+static void
+update_rotation_from_exif_data (DialogData *data,
+				GList      *current_image)
+{
+#ifdef HAVE_LIBEXIF
+	FileData *fd = current_image->data;
+	char     *value = get_exif_tag (fd->path, EXIF_TAG_ORIENTATION);
+
+	if (value != NULL) {
+		if (strcmp (value, "top - left") == 0) {
+			data->rot_type = TRAN_ROTATE_0;
+			data->tran_type = TRAN_NONE;
+		} else if (strcmp (value, "top - right") == 0) {
+			data->rot_type = TRAN_ROTATE_0;
+			data->tran_type = TRAN_MIRROR;
+		} else if (strcmp (value, "bottom - right") == 0) {
+			data->rot_type = TRAN_ROTATE_180;
+			data->tran_type = TRAN_NONE;
+		} else if (strcmp (value, "bottom - left") == 0) {
+			data->rot_type = TRAN_ROTATE_180;
+			data->tran_type = TRAN_MIRROR;
+		} else if (strcmp (value, "left - top") == 0) {
+			data->rot_type = TRAN_ROTATE_90;
+			data->tran_type = TRAN_MIRROR;
+		} else if (strcmp (value, "right - top") == 0) {
+			data->rot_type = TRAN_ROTATE_90;
+			data->tran_type = TRAN_NONE;
+		} else if (strcmp (value, "right - bottom") == 0) {
+			data->rot_type = TRAN_ROTATE_90;
+			data->tran_type = TRAN_FLIP;
+		} else if (strcmp (value, "left - bottom") == 0) {
+			data->rot_type = TRAN_ROTATE_270;
+			data->tran_type = TRAN_NONE;
+		} else {
+			data->rot_type = TRAN_ROTATE_0;
+			data->tran_type = TRAN_NONE;
+		}
+		g_free (value);
+	}
+
+#endif /* HAVE_LIBEXIF */
+}
+
+
+static void
+update_from_exif_data (DialogData *data)
+{
+#ifdef HAVE_LIBEXIF
+	gboolean   from_exif;
+	GdkPixbuf *src_pixbuf, *dest_pixbuf;
+	
+	from_exif = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton));
+	gtk_widget_set_sensitive (data->j_button_box, !from_exif);
+
+	if (! from_exif)
+		return;
+	
+	update_rotation_from_exif_data (data, data->current_image);
+
+	src_pixbuf = data->original_preview;
+	
+	if (src_pixbuf == NULL) 
+		return;
+
+	switch (data->rot_type) {
+	case TRAN_ROTATE_90:
+		dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, FALSE);
+		break;
+	case TRAN_ROTATE_180:
+		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, TRUE);
+		break;
+	case TRAN_ROTATE_270:
+		dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, TRUE);
+		break;
+	default:
+		dest_pixbuf = src_pixbuf;
+		g_object_ref (dest_pixbuf);
+			break;
+	}
+
+	src_pixbuf = dest_pixbuf;
+
+	switch (data->tran_type) {
+	case TRAN_MIRROR:
+		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, FALSE);
+		break;
+	case TRAN_FLIP:
+		dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, FALSE, TRUE);
+		break;
+	default:
+		dest_pixbuf = src_pixbuf;
+		g_object_ref (dest_pixbuf);
+			break;
+	}
+	
+	g_object_unref (src_pixbuf);
+
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
+		
+	g_object_unref (dest_pixbuf);
+
+#endif /* HAVE_LIBEXIF */
+}
+
+
+static void 
+image_loader_done_cb (ImageLoader  *il,
+		      DialogData   *data)
+{
+	GdkPixbuf *pixbuf;
+
+	pixbuf = image_loader_get_pixbuf (il);
+	if (pixbuf != NULL) {
+		if (data->original_preview != NULL)
+			g_object_unref (data->original_preview);
+		data->original_preview = _gdk_pixbuf_scale_keep_aspect_ratio (pixbuf, PREVIEW_SIZE, PREVIEW_SIZE);
+		gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), data->original_preview);
+
+		gtk_widget_set_sensitive (data->j_button_vbox, TRUE);
+		gtk_widget_set_sensitive (data->j_revert_button, TRUE);
+
+		update_from_exif_data (data);
+	}
+}
+
+
+static void 
+image_loader_error_cb (ImageLoader  *il,
+		       DialogData   *data)
+{
+	gtk_widget_set_sensitive (data->j_button_vbox, TRUE);
+	gtk_widget_set_sensitive (data->j_revert_button, TRUE);
 }
 
 
@@ -129,10 +290,12 @@ load_current_image (DialogData *data)
 		return;
 	}
 	
-	gtk_widget_set_sensitive (data->j_button_box, FALSE);
+	gtk_widget_set_sensitive (data->j_button_vbox, FALSE);
+	gtk_widget_set_sensitive (data->j_revert_button, FALSE);
 
 	fd = data->current_image->data;
-	image_viewer_load_image (IMAGE_VIEWER (data->viewer), fd->path);
+	image_loader_set_path (data->loader, fd->path);
+	image_loader_start (data->loader);
 
 	data->rot_type = TRAN_ROTATE_0;
 	data->tran_type = TRAN_NONE;
@@ -191,8 +354,33 @@ swap_fields (ExifContent *content,
 }
 
 
+static ExifShort
+get_next_value_rotation_90 (int value)
+{
+	static ExifShort new_value [8] = {8, 7, 6, 5, 2, 1, 4, 3};
+	return new_value[value - 1];
+}
+
+
+static ExifShort
+get_next_value_mirror (int value)
+{
+	static ExifShort new_value [8] = {2, 1, 4, 3, 8, 7, 6, 5};
+	return new_value[value - 1];
+}
+
+
+static ExifShort
+get_next_value_flip (int value)
+{
+	static ExifShort new_value [8] = {4, 3, 2, 1, 6, 5, 8, 7};
+	return new_value[value - 1];
+}
+
+
 static void
-swap_xy_exif_fields (const char *filename)
+swap_xy_exif_fields (const char *filename,
+		     DialogData *data)
 {
 	JPEGData     *jdata;
 	ExifData     *edata;
@@ -238,12 +426,87 @@ swap_xy_exif_fields (const char *filename)
 }
 
 
+static void
+update_orientation_field (const char *filename,
+			  DialogData *data)
+{
+	JPEGData     *jdata;
+	ExifData     *edata;
+	unsigned int  i;
+	gboolean      orientation_changed = FALSE;
+
+	jdata = jpeg_data_new_from_file (filename);
+	if (jdata == NULL)
+		return;
+
+	edata = jpeg_data_get_exif_data (jdata);
+	if (edata == NULL) {
+		jpeg_data_unref (jdata);
+		return;
+	}
+
+	for (i = 0; i < EXIF_IFD_COUNT; i++) {
+		ExifContent *content = edata->ifd[i];
+		ExifEntry   *entry;
+
+		if ((content == NULL) || (content->count == 0)) 
+			continue;
+
+		entry = exif_content_get_entry (content, EXIF_TAG_ORIENTATION);
+		if (!orientation_changed && (entry != NULL)) {
+			ExifByteOrder byte_order;
+			ExifShort     value;
+
+			byte_order = exif_data_get_byte_order (edata);
+			value = exif_get_short (entry->data, byte_order);
+
+			switch (data->rot_type) {
+			case TRAN_ROTATE_90:
+				value = get_next_value_rotation_90 (value);
+				break;
+			case TRAN_ROTATE_180:
+				value = get_next_value_rotation_90 (value);
+				value = get_next_value_rotation_90 (value);
+				break;
+			case TRAN_ROTATE_270:
+				value = get_next_value_rotation_90 (value);
+				value = get_next_value_rotation_90 (value);
+				value = get_next_value_rotation_90 (value);
+				break;
+			default:
+				break;
+			}
+			
+			switch (data->tran_type) {
+			case TRAN_MIRROR:
+				value = get_next_value_mirror (value);
+				break;
+			case TRAN_FLIP:
+				value = get_next_value_flip (value);
+				break;
+			default:
+				break;
+			}
+
+			exif_set_short (entry->data, byte_order, value);
+
+			orientation_changed = TRUE;
+		}
+	}
+
+	jpeg_data_save_file (jdata, filename);
+
+	exif_data_unref (edata);
+	jpeg_data_unref (jdata);
+}
+
+
 #endif
 
 
 static void
-apply_tran (DialogData *data,
-	    GList      *current_image)
+apply_tranformation_jpeg (DialogData *data,
+			  GList      *current_image)
 {
 	FileData   *fd = current_image->data;
 	int         rot_type = data->rot_type; 
@@ -263,8 +526,6 @@ apply_tran (DialogData *data,
 	if ((rot_type == TRAN_ROTATE_0) && (tran_type == TRAN_NONE))
 		return;
 
-	
-
 	if (rot_type == TRAN_ROTATE_0)
 		tmp1 = g_strdup (fd->path);
 	else {
@@ -402,7 +663,8 @@ apply_tran (DialogData *data,
 	} else {
 #ifdef HAVE_LIBEXIF
 		if ((rot_type == TRAN_ROTATE_90) || (rot_type == TRAN_ROTATE_270))
-			swap_xy_exif_fields (fd->path);
+			swap_xy_exif_fields (fd->path, data);
+		update_orientation_field (fd->path, data);
 #endif
 
 		data->files_changed_list = g_list_prepend (data->files_changed_list, 
@@ -417,6 +679,111 @@ apply_tran (DialogData *data,
 }
 
 
+static void
+apply_transformation_generic (DialogData *data,
+			      GList      *current_image)
+{
+	FileData   *fd = current_image->data;
+	int         rot_type = data->rot_type; 
+	int         tran_type = data->tran_type; 
+	GdkPixbuf  *pixbuf1, *pixbuf2;
+	const char *mime_type;
+
+	if ((rot_type == TRAN_ROTATE_0) && (tran_type == TRAN_NONE))
+		return;
+
+	pixbuf1 = gdk_pixbuf_new_from_file (fd->path, NULL);
+	if (pixbuf1 == NULL)
+		return;
+
+	switch (rot_type) {
+	case TRAN_ROTATE_90:
+		pixbuf2 = _gdk_pixbuf_copy_rotate_90 (pixbuf1, FALSE);
+		break;
+	case TRAN_ROTATE_180:
+		pixbuf2 = _gdk_pixbuf_copy_mirror (pixbuf1, TRUE, TRUE);
+		break;
+	case TRAN_ROTATE_270:
+		pixbuf2 = _gdk_pixbuf_copy_rotate_90 (pixbuf1, TRUE);
+		break;
+	default:
+		pixbuf2 = pixbuf1;
+		g_object_ref (pixbuf2);
+		break;
+	}
+	g_object_unref (pixbuf1);
+
+	switch (tran_type) {
+	case TRAN_MIRROR:
+		pixbuf1 = _gdk_pixbuf_copy_mirror (pixbuf2, TRUE, FALSE);
+		break;
+	case TRAN_FLIP:
+		pixbuf1 = _gdk_pixbuf_copy_mirror (pixbuf2, FALSE, TRUE);
+		break;
+	default:
+		pixbuf1 = pixbuf2;
+		g_object_ref (pixbuf1);
+		break;
+	}
+	g_object_unref (pixbuf2);
+
+	mime_type = gnome_vfs_mime_type_from_name (fd->path);
+	if ((mime_type != NULL) && is_mime_type_writable (mime_type)) {
+		GError      *error = NULL;
+		const char  *image_type = mime_type + 6;
+		if (! _gdk_pixbuf_save (pixbuf1, 
+					fd->path, 
+					image_type, 
+					&error, 
+					NULL))
+			_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->window->app), &error);
+	}
+	g_object_unref (pixbuf1);
+
+	data->files_changed_list = g_list_prepend (data->files_changed_list, 
+						   g_strdup (fd->path));
+}
+
+
+static void
+apply_transformation (DialogData *data,
+		      GList      *current_image)
+{
+	FileData    *fd = current_image->data;
+	char        *dir;
+	struct stat  buf;
+
+	/* Check directory permissions. */
+
+	dir = remove_level_from_path (fd->path);
+	if (access (dir, R_OK | W_OK | X_OK) != 0) {
+		char *utf8_path;
+		utf8_path = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
+		_gtk_error_dialog_run (GTK_WINDOW (data->dialog),
+				       _("You don't have the right permissions to create images in the folder \"%s\""),
+				       utf8_path);
+		g_free (utf8_path);
+		g_free (dir);
+		return;
+	} 
+	g_free (dir);
+
+	/**/
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton)))
+		update_rotation_from_exif_data (data, current_image);
+
+	stat (fd->path, &buf);
+
+	if (image_is_jpeg (fd->path)) 
+		apply_tranformation_jpeg (data, current_image);
+	else 
+		apply_transformation_generic (data, current_image);
+
+	chmod (fd->path, buf.st_mode);
+}
+
+
 static void
 ok_clicked (GtkWidget  *button, 
 	    DialogData *data)
@@ -456,12 +823,12 @@ ok_clicked (GtkWidget  *button,
 
 			_gtk_label_set_filename_text (GTK_LABEL (label), fd->name);
 			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (bar),
-						       (gdouble) i / n);
+						       (gdouble) (i + 1) / n);
 
 			while (gtk_events_pending())
 				gtk_main_iteration();
 
-			apply_tran (data, scan);
+			apply_transformation (data, scan);
 
 			i++;
 		}
@@ -472,7 +839,7 @@ ok_clicked (GtkWidget  *button,
 		gtk_widget_destroy (data->dialog);
 
 	} else {
-		apply_tran (data, data->current_image);
+		apply_transformation (data, data->current_image);
 		load_next_image (data);
 	}
 }
@@ -493,7 +860,13 @@ static void
 revert_clicked (GtkWidget  *button, 
 		DialogData *data)
 {
-	load_current_image (data);
+	data->rot_type = TRAN_ROTATE_0;
+	data->tran_type = TRAN_NONE;
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->j_from_exif_checkbutton), FALSE);
+
+	if (data->original_preview != NULL)
+		gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), data->original_preview);
 }
 
 
@@ -501,9 +874,8 @@ static void
 rot90_clicked (GtkWidget  *button, 
 	       DialogData *data)
 {
-	ImageViewer  *viewer = IMAGE_VIEWER (data->viewer);
-	GdkPixbuf    *src_pixbuf;
-	GdkPixbuf    *dest_pixbuf;
+	GdkPixbuf *src_pixbuf;
+	GdkPixbuf *dest_pixbuf;
 
 	if (data->tran_type == TRAN_NONE)
 		data->rot_type = get_next_rot (data->rot_type);
@@ -513,13 +885,13 @@ rot90_clicked (GtkWidget  *button,
 		data->rot_type = get_next_rot (data->rot_type);
 	}
 
-	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
 
 	if (src_pixbuf == NULL) 
 		return;
 
 	dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, FALSE);
-	image_viewer_set_pixbuf (viewer, dest_pixbuf);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
 
 	if (dest_pixbuf != NULL) 
 		g_object_unref (dest_pixbuf);
@@ -530,9 +902,8 @@ static void
 rot270_clicked (GtkWidget  *button, 
 		DialogData *data)
 {
-	ImageViewer  *viewer = IMAGE_VIEWER (data->viewer);
-	GdkPixbuf    *src_pixbuf;
-	GdkPixbuf    *dest_pixbuf;
+	GdkPixbuf *src_pixbuf;
+	GdkPixbuf *dest_pixbuf;
 
 	if (data->tran_type == TRAN_NONE) {
 		data->rot_type = get_next_rot (data->rot_type);
@@ -541,13 +912,13 @@ rot270_clicked (GtkWidget  *button,
 	} else
 		data->rot_type = get_next_rot (data->rot_type);
 
-	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
 
 	if (src_pixbuf == NULL) 
 		return;
 
 	dest_pixbuf = _gdk_pixbuf_copy_rotate_90 (src_pixbuf, TRUE);
-	image_viewer_set_pixbuf (viewer, dest_pixbuf);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
 
 	if (dest_pixbuf != NULL) 
 		g_object_unref (dest_pixbuf);
@@ -558,11 +929,10 @@ static void
 mirror_clicked (GtkWidget  *button, 
 		DialogData *data)
 {
-	ImageViewer *viewer = IMAGE_VIEWER (data->viewer);
-	GdkPixbuf   *src_pixbuf;
-	GdkPixbuf   *dest_pixbuf;
+	GdkPixbuf *src_pixbuf;
+	GdkPixbuf *dest_pixbuf;
 
-	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
 
 	if (src_pixbuf == NULL) 
 		return;
@@ -579,7 +949,7 @@ mirror_clicked (GtkWidget  *button,
 		data->tran_type = TRAN_MIRROR;
 	
 	dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, TRUE, FALSE);
-	image_viewer_set_pixbuf (viewer, dest_pixbuf);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
 	
 	if (dest_pixbuf != NULL) 
 		g_object_unref (dest_pixbuf);
@@ -590,9 +960,8 @@ static void
 flip_clicked (GtkWidget  *button, 
 	      DialogData *data)
 {
-	ImageViewer  *viewer = IMAGE_VIEWER (data->viewer);
-	GdkPixbuf    *src_pixbuf;
-	GdkPixbuf    *dest_pixbuf;
+	GdkPixbuf *src_pixbuf;
+	GdkPixbuf *dest_pixbuf;
 
 	if (data->tran_type == TRAN_MIRROR) {
 		data->tran_type = TRAN_NONE;
@@ -605,13 +974,13 @@ flip_clicked (GtkWidget  *button,
 	else
 		data->tran_type = TRAN_FLIP;
 
-	src_pixbuf = image_viewer_get_current_pixbuf (viewer);
+	src_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (data->j_preview_image));
 
 	if (src_pixbuf == NULL) 
 		return;
 
 	dest_pixbuf = _gdk_pixbuf_copy_mirror (src_pixbuf, FALSE, TRUE);
-	image_viewer_set_pixbuf (viewer, dest_pixbuf);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->j_preview_image), dest_pixbuf);
 
 	if (dest_pixbuf != NULL) 
 		g_object_unref (dest_pixbuf);
@@ -651,6 +1020,14 @@ help_cb (GtkWidget  *widget,
 }
 
 
+static void
+from_exif_toggled_cb (GtkToggleButton *button,
+		      DialogData      *data)
+{
+	update_from_exif_data (data);
+}
+
+
 void
 dlg_jpegtran (GThumbWindow *window)
 {
@@ -664,7 +1041,8 @@ dlg_jpegtran (GThumbWindow *window)
 	GtkWidget   *j_help_button;
 	GtkWidget   *j_cancel_button;
 	GtkWidget   *j_ok_button;
-	GList       *list, *scan;
+	GtkWidget   *reset_image;
+	GList       *list;
 
 	list = gth_file_list_get_selection_as_fd (window->file_list);
 	if (list == NULL) {
@@ -672,26 +1050,6 @@ dlg_jpegtran (GThumbWindow *window)
 		return;
 	}
 
-	/* remove non jpeg images */
-
-	for (scan = list; scan; ) {
-		FileData *fd = scan->data;
-		GList    *next = scan->next;
-
-		if (! image_is_jpeg (fd->path)) {
-			list = g_list_remove_link (list, scan);
-			g_list_free (scan);
-			file_data_unref (fd);
-		}
-
-		scan = next;
-	}
-
-	if (list == NULL) {
-		g_warning ("No JPEG image selected");
-		return;
-	}
-
 	/**/
 
 	data = g_new0 (DialogData, 1);
@@ -714,6 +1072,9 @@ dlg_jpegtran (GThumbWindow *window)
 	data->dialog = glade_xml_get_widget (data->gui, "jpeg_rotate_dialog");
 	data->j_apply_to_all_checkbutton = glade_xml_get_widget (data->gui, "j_apply_to_all_checkbutton");
 	data->j_button_box = glade_xml_get_widget (data->gui, "j_button_box");
+	data->j_button_vbox = glade_xml_get_widget (data->gui, "j_button_vbox");
+	data->j_revert_button = glade_xml_get_widget (data->gui, "j_revert_button");
+	data->j_preview_image = glade_xml_get_widget (data->gui, "j_preview_image");
 
 	j_image_vbox = glade_xml_get_widget (data->gui, "j_image_vbox");
 	j_revert_button = glade_xml_get_widget (data->gui, "j_revert_button");
@@ -722,27 +1083,17 @@ dlg_jpegtran (GThumbWindow *window)
 	j_v_flip_button = glade_xml_get_widget (data->gui, "j_v_flip_button");
 	j_h_flip_button = glade_xml_get_widget (data->gui, "j_h_flip_button");
 
+	data->j_from_exif_checkbutton = glade_xml_get_widget (data->gui, "j_from_exif_checkbutton");
+
 	j_help_button = glade_xml_get_widget (data->gui, "j_help_button");
 	j_cancel_button = glade_xml_get_widget (data->gui, "j_cancel_button");
 	j_ok_button = glade_xml_get_widget (data->gui, "j_ok_button");
 
-	/* image viewer */
-
-	data->viewer = image_viewer_new ();
-	image_viewer_size (IMAGE_VIEWER (data->viewer), PREVIEW_SIZE, PREVIEW_SIZE);
-	gtk_container_add (GTK_CONTAINER (j_image_vbox), data->viewer);
-	image_viewer_set_zoom_change (IMAGE_VIEWER (data->viewer), GTH_ZOOM_CHANGE_FIT_IF_LARGER);
-	image_viewer_set_zoom_quality (IMAGE_VIEWER (data->viewer),
-				       pref_get_zoom_quality ());
-	image_viewer_set_check_type (IMAGE_VIEWER (data->viewer),
-				     image_viewer_get_check_type (IMAGE_VIEWER (window->viewer)));
-	image_viewer_set_check_size (IMAGE_VIEWER (data->viewer),
-				     image_viewer_get_check_size (IMAGE_VIEWER (window->viewer)));
-	image_viewer_set_transp_type (IMAGE_VIEWER (data->viewer),
-				      image_viewer_get_transp_type (IMAGE_VIEWER (window->viewer)));
-
 	/* Set widgets data. */
 
+	reset_image = glade_xml_get_widget (data->gui, "j_reset_image");
+	gtk_image_set_from_stock (GTK_IMAGE (reset_image), GTHUMB_STOCK_RESET, GTK_ICON_SIZE_MENU);
+
 	add_image_to_button (j_rot_90_button, rotate_90_24_rgba);
 	add_image_to_button (j_rot_270_button, rotate_270_24_rgba);
 	add_image_to_button (j_v_flip_button, mirror_24_rgba);
@@ -751,6 +1102,10 @@ dlg_jpegtran (GThumbWindow *window)
 	gtk_widget_set_sensitive (data->j_apply_to_all_checkbutton,
 				  data->file_list->next != NULL);
 
+#ifndef HAVE_LIBEXIF
+	gtk_widget_set_sensitive (data->j_from_exif_checkbutton, FALSE);
+#endif /* ! HAVE_LIBEXIF */
+
 	/* Set the signals handlers. */
 	
 	g_signal_connect (G_OBJECT (data->dialog),
@@ -790,10 +1145,20 @@ dlg_jpegtran (GThumbWindow *window)
 			  "clicked",
 			  G_CALLBACK (flip_clicked),
 			  data);
+	g_signal_connect (G_OBJECT (data->j_from_exif_checkbutton),
+			  "toggled",
+			  G_CALLBACK (from_exif_toggled_cb),
+			  data);
 
-	g_signal_connect (G_OBJECT (data->viewer),
-			  "image_loaded",
-			  G_CALLBACK (image_loaded_cb),
+	data->loader = (ImageLoader*)image_loader_new (NULL, FALSE);
+
+	g_signal_connect (G_OBJECT (data->loader),
+			  "image_done",
+			  G_CALLBACK (image_loader_done_cb),
+			  data);
+	g_signal_connect (G_OBJECT (data->loader),
+			  "image_error",
+			  G_CALLBACK (image_loader_error_cb),
 			  data);
 
 	/* Run dialog. */
